@@ -7,8 +7,6 @@
 # http://opensource.org/licenses/MIT>, at your option. This file may not be
 # copied, modified, or distributed except according to those terms.
 
-import binascii
-import getpass
 import json
 import os
 import platform
@@ -23,11 +21,11 @@ if "linux" in platform.platform().lower():
     import fcntl
 
 # @fixme: 1st layer `nkfido2` lower layer `fido2` not to be used here !
-from cryptography.hazmat.primitives import hashes
 from fido2.client import ClientError as Fido2ClientError
 from fido2.ctap import CtapError
 from fido2.ctap1 import ApduError
-from fido2.ctap2 import PinProtocolV1 as PIN
+from fido2.ctap2 import Ctap2
+from fido2.ctap2.pin import ClientPin
 
 import pynitrokey
 import pynitrokey.fido2 as nkfido2
@@ -515,8 +513,10 @@ def change_pin(serial):
         )
     try:
         # @fixme: move this (function) into own fido2-client-class
-        client = nkfido2.find(serial).client
-        client.client_pin.change_pin(old_pin, new_pin)
+        dev = nkfido2.find(serial)
+        client = dev.client
+        client_pin = ClientPin(dev.ctap2)
+        client_pin.change_pin(old_pin, new_pin)
         local_print("done - please use new pin to verify key")
 
     except Exception as e:
@@ -544,8 +544,10 @@ def set_pin(serial):
         )
     try:
         # @fixme: move this (function) into own fido2-client-class
-        client = nkfido2.find(serial).client
-        client.client_pin.set_pin(new_pin)
+        dev = nkfido2.find(serial)
+        client = dev.client
+        client_pin = ClientPin(dev.ctap2)
+        client_pin.set_pin(new_pin)
         local_print("done - please use new pin to verify key")
 
     except Exception as e:
